@@ -22,7 +22,8 @@
 #include "omap.h"
 #include "common/plat.h"
 #include "../libpcsxcore/misc.h"
-#include "../libpcsxcore/new_dynarec/new_dynarec.h"
+#include "../libpcsxcore/cdrom.h"
+#include "../libpcsxcore/psemu_plugin_defs.h"
 #include "revision.h"
 
 #define MENU_X2 1
@@ -36,6 +37,7 @@ typedef enum
 	MA_MAIN_LOAD_STATE,
 	MA_MAIN_RESET_GAME,
 	MA_MAIN_LOAD_ROM,
+	MA_MAIN_SWAP_CD,
 	MA_MAIN_RUN_BIOS,
 	MA_MAIN_CONTROLS,
 	MA_MAIN_CREDITS,
@@ -60,9 +62,10 @@ enum {
 };
 
 static int last_psx_w, last_psx_h, last_psx_bpp;
-static int scaling, filter, state_slot, cpu_clock, cpu_clock_st;
+static int scaling, filter, cpu_clock, cpu_clock_st;
 static char rom_fname_reload[MAXPATHLEN];
 static char last_selected_fname[MAXPATHLEN];
+static int region, in_type_sel;
 int g_opts;
 
 // from softgpu plugin
@@ -102,28 +105,16 @@ void emu_make_path(char *buff, const char *end, int size)
 
 static int emu_check_save_file(int slot)
 {
-	char fname[MAXPATHLEN];
-	int ret;
-
-	ret = get_state_filename(fname, sizeof(fname), slot);
-	if (ret != 0)
-		return 0;
-
-	ret = CheckState(fname);
+	int ret = emu_check_state(slot);
 	return ret == 0 ? 1 : 0;
 }
 
-static int emu_save_load_game(int load, int sram)
+static int emu_save_load_game(int load, int unused)
 {
-	char fname[MAXPATHLEN];
 	int ret;
 
-	ret = get_state_filename(fname, sizeof(fname), state_slot);
-	if (ret != 0)
-		return 0;
-
 	if (load) {
-		ret = LoadState(fname);
+		ret = emu_load_state(state_slot);
 
 		// reflect hle/bios mode from savestate
 		if (Config.HLE)
@@ -133,26 +124,48 @@ static int emu_save_load_game(int load, int sram)
 			bios_sel = 1;
 	}
 	else
-		ret = SaveState(fname);
+		ret = emu_save_state(state_slot);
 
 	return ret;
 }
 
+// propagate menu settings to the emu vars
+static void menu_sync_config(void)
+{
+	Config.PsxAuto = 1;
+	if (region > 0) {
+		Config.PsxAuto = 0;
+		Config.PsxType = region - 1;
+	}
+	in_type = in_type_sel ? PSE_PAD_TYPE_ANALOGPAD : PSE_PAD_TYPE_STANDARD;
+
+	pl_frame_interval = Config.PsxType ? 20000 : 16667;
+	// used by P.E.Op.S. frameskip code
+	fFrameRateHz = Config.PsxType ? 50.0f : 59.94f;
+	dwFrameRateTicks = (100000*100 / (unsigned long)(fFrameRateHz*100));
+}
+
 static void menu_set_defconfig(void)
 {
 	g_opts = 0;
 	scaling = SCALE_4_3;
 
+	region = 0;
+	in_type_sel = 0;
 	Config.Xa = Config.Cdda = Config.Sio =
 	Config.SpuIrq = Config.RCntFix = Config.VSyncWA = 0;
 
-	iUseDither = UseFrameSkip = 0;
+	iUseDither = 0;
+	UseFrameSkip = 1;
 	dwActFixes = 1<<7;
 
 	iUseReverb = 2;
 	iUseInterpolation = 1;
-	iXAPitch = iSPUIRQWait = 0;
+	iXAPitch = 0;
+	iSPUIRQWait = 1;
 	iUseTimer = 2;
+
+	menu_sync_config();
 }
 
 #define CE_CONFIG_STR(val) \
@@ -167,6 +180,10 @@ static void menu_set_defconfig(void)
 #define CE_INTVAL(val) \
 	{ #val, sizeof(val), &val }
 
+// 'versioned' var, used when defaults change
+#define CE_INTVAL_V(val, ver) \
+	{ #val #ver, sizeof(val), &val }
+
 static const struct {
 	const char *name;
 	size_t len;
@@ -179,7 +196,6 @@ static const struct {
 	CE_CONFIG_VAL(Xa),
 	CE_CONFIG_VAL(Sio),
 	CE_CONFIG_VAL(Mdec),
-	CE_CONFIG_VAL(PsxAuto),
 	CE_CONFIG_VAL(Cdda),
 	CE_CONFIG_VAL(Debug),
 	CE_CONFIG_VAL(PsxOut),
@@ -187,7 +203,7 @@ static const struct {
 	CE_CONFIG_VAL(RCntFix),
 	CE_CONFIG_VAL(VSyncWA),
 	CE_CONFIG_VAL(Cpu),
-	CE_CONFIG_VAL(PsxType),
+	CE_INTVAL(region),
 	CE_INTVAL(scaling),
 	CE_INTVAL(g_layer_x),
 	CE_INTVAL(g_layer_y),
@@ -197,13 +213,14 @@ static const struct {
 	CE_INTVAL(state_slot),
 	CE_INTVAL(cpu_clock),
 	CE_INTVAL(g_opts),
+	CE_INTVAL(in_type_sel),
 	CE_INTVAL(iUseDither),
 	CE_INTVAL(UseFrameSkip),
 	CE_INTVAL(dwActFixes),
 	CE_INTVAL(iUseReverb),
-	CE_INTVAL(iUseInterpolation),
 	CE_INTVAL(iXAPitch),
-	CE_INTVAL(iSPUIRQWait),
+	CE_INTVAL_V(iUseInterpolation, 2),
+	CE_INTVAL_V(iSPUIRQWait, 2),
 	CE_INTVAL(iUseTimer),
 };
 
@@ -229,6 +246,8 @@ static void make_cfg_fname(char *buf, size_t size, int is_game)
 		snprintf(buf, size, "." PCSX_DOT_DIR "%s", cfgfile_basename);
 }
 
+static void keys_write_all(FILE *f);
+
 static int menu_write_config(int is_game)
 {
 	char cfgfile[MAXPATHLEN];
@@ -267,7 +286,9 @@ static int menu_write_config(int is_game)
 	if (!is_game)
 		fprintf(f, "lastcdimg = %s\n", last_selected_fname);
 
+	keys_write_all(f);
 	fclose(f);
+
 	return 0;
 }
 
@@ -283,6 +304,8 @@ static void parse_str_val(char *cval, const char *src)
 		*tmp = 0;
 }
 
+static void keys_load_all(const char *cfg);
+
 static int menu_load_config(int is_game)
 {
 	char cfgfile[MAXPATHLEN];
@@ -363,6 +386,8 @@ static int menu_load_config(int is_game)
 		}
 	}
 
+	menu_sync_config();
+
 	// sync plugins
 	for (i = bios_sel = 0; bioses[i] != NULL; i++)
 		if (strcmp(Config.Bios, bioses[i]) == 0)
@@ -376,6 +401,7 @@ static int menu_load_config(int is_game)
 		if (strcmp(Config.Spu, spu_plugins[i]) == 0)
 			{ spu_plugsel = i; break; }
 
+	keys_load_all(cfg);
 	ret = 0;
 fail_read:
 	free(cfg);
@@ -623,6 +649,8 @@ me_bind_action me_ctrl_actions[] =
 	{ "R1      ", 1 << DKEY_R1 },
 	{ "L2      ", 1 << DKEY_L2 },
 	{ "R2      ", 1 << DKEY_R2 },
+	{ "L3      ", 1 << DKEY_L3 },
+	{ "R3      ", 1 << DKEY_R3 },
 	{ "START   ", 1 << DKEY_START },
 	{ "SELECT  ", 1 << DKEY_SELECT },
 	{ NULL,       0 }
@@ -630,16 +658,186 @@ me_bind_action me_ctrl_actions[] =
 
 me_bind_action emuctrl_actions[] =
 {
-/*
-	{ "Load State       ", PEV_STATE_LOAD },
-	{ "Save State       ", PEV_STATE_SAVE },
-	{ "Prev Save Slot   ", PEV_SSLOT_PREV },
-	{ "Next Save Slot   ", PEV_SSLOT_NEXT },
-*/
-	{ "Enter Menu       ", PEV_MENU },
+	{ "Save State       ", 1 << SACTION_SAVE_STATE },
+	{ "Load State       ", 1 << SACTION_LOAD_STATE },
+	{ "Prev Save Slot   ", 1 << SACTION_PREV_SSLOT },
+	{ "Next Save Slot   ", 1 << SACTION_NEXT_SSLOT },
+	{ "Toggle Frameskip ", 1 << SACTION_TOGGLE_FSKIP },
+	{ "Enter Menu       ", 1 << SACTION_ENTER_MENU },
 	{ NULL,                0 }
 };
 
+static char *mystrip(char *str)
+{
+	int i, len;
+
+	len = strlen(str);
+	for (i = 0; i < len; i++)
+		if (str[i] != ' ') break;
+	if (i > 0) memmove(str, str + i, len - i + 1);
+
+	len = strlen(str);
+	for (i = len - 1; i >= 0; i--)
+		if (str[i] != ' ') break;
+	str[i+1] = 0;
+
+	return str;
+}
+
+static void get_line(char *d, size_t size, const char *s)
+{
+	const char *pe;
+	size_t len;
+
+	for (pe = s; *pe != '\r' && *pe != '\n' && *pe != 0; pe++)
+		;
+	len = pe - s;
+	if (len > size - 1)
+		len = size - 1;
+	strncpy(d, s, len);
+	d[len] = 0;
+
+	mystrip(d);
+}
+
+static void keys_write_all(FILE *f)
+{
+	int d;
+
+	for (d = 0; d < IN_MAX_DEVS; d++)
+	{
+		const int *binds = in_get_dev_binds(d);
+		const char *name = in_get_dev_name(d, 0, 0);
+		int k, count = 0;
+
+		if (binds == NULL || name == NULL)
+			continue;
+
+		fprintf(f, "binddev = %s\n", name);
+		in_get_config(d, IN_CFG_BIND_COUNT, &count);
+
+		for (k = 0; k < count; k++)
+		{
+			int i, kbinds, mask;
+			char act[32];
+
+			act[0] = act[31] = 0;
+			name = in_get_key_name(d, k);
+
+			kbinds = binds[IN_BIND_OFFS(k, IN_BINDTYPE_PLAYER12)];
+			for (i = 0; kbinds && i < ARRAY_SIZE(me_ctrl_actions) - 1; i++) {
+				mask = me_ctrl_actions[i].mask;
+				if (mask & kbinds) {
+					strncpy(act, me_ctrl_actions[i].name, 31);
+					fprintf(f, "bind %s = player1 %s\n", name, mystrip(act));
+					kbinds &= ~mask;
+				}
+				mask = me_ctrl_actions[i].mask << 16;
+				if (mask & kbinds) {
+					strncpy(act, me_ctrl_actions[i].name, 31);
+					fprintf(f, "bind %s = player2 %s\n", name, mystrip(act));
+					kbinds &= ~mask;
+				}
+			}
+
+			kbinds = binds[IN_BIND_OFFS(k, IN_BINDTYPE_EMU)];
+			for (i = 0; kbinds && i < ARRAY_SIZE(emuctrl_actions) - 1; i++) {
+				mask = emuctrl_actions[i].mask;
+				if (mask & kbinds) {
+					strncpy(act, emuctrl_actions[i].name, 31);
+					fprintf(f, "bind %s = %s\n", name, mystrip(act));
+					kbinds &= ~mask;
+				}
+			}
+		}
+	}
+}
+
+static int parse_bind_val(const char *val, int *type)
+{
+	int i;
+
+	*type = IN_BINDTYPE_NONE;
+	if (val[0] == 0)
+		return 0;
+	
+	if (strncasecmp(val, "player", 6) == 0)
+	{
+		int player, shift = 0;
+		player = atoi(val + 6) - 1;
+
+		if ((unsigned int)player > 1)
+			return -1;
+		if (player == 1)
+			shift = 16;
+
+		*type = IN_BINDTYPE_PLAYER12;
+		for (i = 0; me_ctrl_actions[i].name != NULL; i++) {
+			if (strncasecmp(me_ctrl_actions[i].name, val + 8, strlen(val + 8)) == 0)
+				return me_ctrl_actions[i].mask << shift;
+		}
+	}
+	for (i = 0; emuctrl_actions[i].name != NULL; i++) {
+		if (strncasecmp(emuctrl_actions[i].name, val, strlen(val)) == 0) {
+			*type = IN_BINDTYPE_EMU;
+			return emuctrl_actions[i].mask;
+		}
+	}
+
+	return -1;
+}
+
+static void keys_load_all(const char *cfg)
+{
+	char dev[256], key[128], *act;
+	const char *p;
+	int bind, bindtype;
+	int dev_id;
+
+	p = cfg;
+	while (p != NULL && (p = strstr(p, "binddev = ")) != NULL) {
+		p += 10;
+
+		get_line(dev, sizeof(dev), p);
+		dev_id = in_config_parse_dev(dev);
+		if (dev_id < 0) {
+			printf("input: can't handle dev: %s\n", dev);
+			continue;
+		}
+
+		in_unbind_all(dev_id, -1, -1);
+		while ((p = strstr(p, "bind"))) {
+			if (strncmp(p, "binddev = ", 10) == 0)
+				break;
+
+			p += 4;
+			if (*p != ' ') {
+				printf("input: parse error: %16s..\n", p);
+				continue;
+			}
+
+			get_line(key, sizeof(key), p);
+			act = strchr(key, '=');
+			if (act == NULL) {
+				printf("parse failed: %16s..\n", p);
+				continue;
+			}
+			*act = 0;
+			act++;
+			mystrip(key);
+			mystrip(act);
+
+			bind = parse_bind_val(act, &bindtype);
+			if (bind != -1 && bind != 0) {
+				//printf("bind #%d '%s' %08x (%s)\n", dev_id, key, bind, act);
+				in_config_bind_key(dev_id, key, bind, bindtype);
+			}
+			else
+				lprintf("config: unhandled action \"%s\"\n", act);
+		}
+	}
+}
+
 static int key_config_loop_wrap(int id, int keys)
 {
 	switch (id) {
@@ -691,13 +889,17 @@ static int mh_savecfg(int id, int keys)
 	return 1;
 }
 
+static const char *men_in_type_sel[] = { "Standard (SCPH-1080)", "Analog (SCPH-1150)", NULL };
+
 static menu_entry e_menu_keyconfig[] =
 {
 	mee_handler_id("Player 1",          MA_CTRL_PLAYER1,    key_config_loop_wrap),
 	mee_handler_id("Player 2",          MA_CTRL_PLAYER2,    key_config_loop_wrap),
 	mee_handler_id("Emulator controls", MA_CTRL_EMU,        key_config_loop_wrap),
-//	mee_cust_nosave("Save global config",       MA_OPT_SAVECFG, mh_savecfg, mgn_saveloadcfg),
-//	mee_cust_nosave("Save cfg for loaded game", MA_OPT_SAVECFG_GAME, mh_savecfg, mgn_saveloadcfg),
+	mee_label     (""),
+	mee_enum      ("Controller",        0, in_type_sel,     men_in_type_sel),
+	mee_cust_nosave("Save global config",       MA_OPT_SAVECFG,      mh_savecfg, mgn_saveloadcfg),
+	mee_cust_nosave("Save cfg for loaded game", MA_OPT_SAVECFG_GAME, mh_savecfg, mgn_saveloadcfg),
 	mee_label     (""),
 	mee_label     ("Input devices:"),
 	mee_label_mk  (MA_CTRL_DEV_FIRST, mgn_dev_name),
@@ -831,7 +1033,7 @@ static int menu_loop_plugin_gpu(int id, int keys)
 
 static const char *men_spu_reverb[] = { "Off", "Fake", "On", NULL };
 static const char *men_spu_interp[] = { "None", "Simple", "Gaussian", "Cubic", NULL };
-static const char h_spu_irq_wait[]  = "Wait for CPU; only useful for some games, may cause glitches";
+static const char h_spu_irq_wait[]  = "Wait for CPU (recommended set to ON)";
 static const char h_spu_thread[]    = "Run sound emulation in main thread (recommended)";
 
 static menu_entry e_menu_plugin_spu[] =
@@ -887,7 +1089,7 @@ static int menu_loop_plugin_options(int id, int keys)
 
 // ------------ adv options menu ------------
 
-static const char h_cfg_cpul[]   = "Shows CPU usage in %%";
+static const char h_cfg_cpul[]   = "Shows CPU usage in %";
 static const char h_cfg_fl[]     = "Frame Limiter keeps the game from running too fast";
 static const char h_cfg_xa[]     = "Disables XA sound, which can sometimes improve performance";
 static const char h_cfg_cdda[]   = "Disable CD Audio for a performance boost\n"
@@ -929,7 +1131,7 @@ static int mh_restore_defaults(int id, int keys)
 	return 1;
 }
 
-static const char *men_region[]       = { "NTSC", "PAL", NULL };
+static const char *men_region[]       = { "Auto", "NTSC", "PAL", NULL };
 /*
 static const char *men_confirm_save[] = { "OFF", "writes", "loads", "both", NULL };
 static const char h_confirm_save[]    = "Ask for confirmation when overwriting save,\n"
@@ -944,7 +1146,7 @@ static menu_entry e_menu_options[] =
 //	mee_enum_h    ("Confirm savestate",        0, dummy, men_confirm_save, h_confirm_save),
 	mee_onoff     ("Frameskip",                0, UseFrameSkip, 1),
 	mee_onoff     ("Show FPS",                 0, g_opts, OPT_SHOWFPS),
-	mee_enum      ("Region",                   0, Config.PsxType, men_region),
+	mee_enum      ("Region",                   0, region, men_region),
 	mee_range     ("CPU clock",                MA_OPT_CPU_CLOCKS, cpu_clock, 20, 5000),
 	mee_handler   ("[Display]",                menu_loop_gfx_options),
 	mee_handler   ("[BIOS/Plugins]",           menu_loop_plugin_options),
@@ -1001,7 +1203,8 @@ static void draw_frame_main(void)
 {
 	if (CdromId[0] != 0) {
 		char buff[64];
-		snprintf(buff, sizeof(buff), "%.32s/%.9s", get_cd_label(), CdromId);
+		snprintf(buff, sizeof(buff), "%.32s/%.9s (running as %s)",
+			 get_cd_label(), CdromId, Config.PsxType ? "PAL" : "NTSC");
 		smalltext_out16(4, 1, buff, 0x105f);
 	}
 }
@@ -1016,10 +1219,10 @@ const char *plat_get_credits(void)
 	return	"PCSX-ReARMed\n\n"
 		"(C) 1999-2003 PCSX Team\n"
 		"(C) 2005-2009 PCSX-df Team\n"
-		"(C) 2009-2010 PCSX-Reloaded Team\n\n"
+		"(C) 2009-2011 PCSX-Reloaded Team\n\n"
 		"GPU and SPU code by Pete Bernert\n"
 		"  and the P.E.Op.S. team\n"
-		"ARM recompiler (C) 2009-2010 Ari64\n"
+		"ARM recompiler (C) 2009-2011 Ari64\n"
 		"PCSX4ALL plugins by PCSX4ALL team\n"
 		"  Chui, Franxis, Unai\n\n"
 		"integration, optimization and\n"
@@ -1134,6 +1337,36 @@ static int romsel_run(void)
 	return 0;
 }
 
+static int swap_cd_image(void)
+{
+	char *fname;
+
+	fname = menu_loop_romsel(last_selected_fname, sizeof(last_selected_fname));
+	if (fname == NULL)
+		return -1;
+
+	printf("selected file: %s\n", fname);
+
+	CdromId[0] = '\0';
+	CdromLabel[0] = '\0';
+
+	set_cd_image(fname);
+	if (ReloadCdromPlugin() < 0) {
+		me_update_msg("failed to load cdr plugin");
+		return -1;
+	}
+	if (CDR_open() < 0) {
+		me_update_msg("failed to open cdr plugin");
+		return -1;
+	}
+
+	SetCdOpenCaseTime(time(NULL) + 2);
+	LidInterrupt();
+
+	strcpy(last_selected_fname, rom_fname_reload);
+	return 0;
+}
+
 static int main_menu_handler(int id, int keys)
 {
 	switch (id)
@@ -1158,6 +1391,10 @@ static int main_menu_handler(int id, int keys)
 		if (romsel_run() == 0)
 			return 1;
 		break;
+	case MA_MAIN_SWAP_CD:
+		if (swap_cd_image() == 0)
+			return 1;
+		break;
 	case MA_MAIN_RUN_BIOS:
 		if (run_bios() == 0)
 			return 1;
@@ -1186,6 +1423,7 @@ static menu_entry e_menu_main[] =
 	mee_handler_id("Load State",         MA_MAIN_LOAD_STATE,  main_menu_handler),
 	mee_handler_id("Reset game",         MA_MAIN_RESET_GAME,  main_menu_handler),
 	mee_handler_id("Load CD image",      MA_MAIN_LOAD_ROM,    main_menu_handler),
+	mee_handler_id("Change CD image",    MA_MAIN_SWAP_CD,     main_menu_handler),
 	mee_handler_id("Run BIOS",           MA_MAIN_RUN_BIOS,    main_menu_handler),
 	mee_handler   ("Options",            menu_loop_options),
 	mee_handler   ("Controls",           menu_loop_keyconfig),
@@ -1208,6 +1446,7 @@ void menu_loop(void)
 	me_enable(e_menu_main, MA_MAIN_SAVE_STATE,  ready_to_go && CdromId[0]);
 	me_enable(e_menu_main, MA_MAIN_LOAD_STATE,  ready_to_go && CdromId[0]);
 	me_enable(e_menu_main, MA_MAIN_RESET_GAME,  ready_to_go);
+	me_enable(e_menu_main, MA_MAIN_SWAP_CD,  ready_to_go);
 	me_enable(e_menu_main, MA_MAIN_RUN_BIOS, bios_sel != 0);
 
 	in_set_config_int(0, IN_CFG_BLOCKING, 1);
@@ -1413,7 +1652,6 @@ void menu_prepare_emu(void)
 	}
 	apply_filter(filter);
 	apply_cpu_clock();
-	stop = 0;
 
 	psxCpu = (Config.Cpu == CPU_INTERPRETER) ? &psxInt : &psxRec;
 	if (psxCpu != prev_cpu)
@@ -1425,7 +1663,7 @@ void menu_prepare_emu(void)
 	if (Config.Cdda)
 		CDR_stop();
 
-	pl_frame_interval = Config.PsxType ? 20000 : 16667;
+	menu_sync_config();
 
 	if (GPU_open != NULL) {
 		int ret = GPU_open(&gpuDisp, "PCSX", NULL);
