@@ -1,5 +1,5 @@
 /*
- * (C) notaz, 2010
+ * (C) notaz, 2010-2011
  *
  * This work is licensed under the terms of the GNU GPLv2 or later.
  * See the COPYING file in the top-level directory.
@@ -9,11 +9,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <stdint.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <pthread.h>
 
 #include "plugin_lib.h"
 #include "linux/fbdev.h"
@@ -21,15 +23,21 @@
 #include "common/input.h"
 #include "omap.h"
 #include "menu.h"
+#include "main.h"
 #include "pcnt.h"
 #include "../libpcsxcore/new_dynarec/new_dynarec.h"
+#include "../libpcsxcore/psemu_plugin_defs.h"
 
 void *pl_fbdev_buf;
 int pl_frame_interval;
-int keystate;
+int in_type, in_keystate, in_a1[2], in_a2[2];
 static int pl_fbdev_w, pl_fbdev_h, pl_fbdev_bpp;
 static int flip_cnt, vsync_cnt, flips_per_sec, tick_per_sec;
 static float vsps_cur;
+static int plugin_skip_advice;
+// P.E.Op.S.
+extern int UseFrameSkip;
+extern float fps_skip;
 
 static int get_cpu_ticks(void)
 {
@@ -51,6 +59,12 @@ static int get_cpu_ticks(void)
 	return ret;
 }
 
+static void print_hud(void)
+{
+	if (pl_fbdev_bpp == 16)
+		pl_text_out16(2, pl_fbdev_h - 10, "%s", hud_msg);
+}
+
 static void print_fps(void)
 {
 	if (pl_fbdev_bpp == 16)
@@ -91,8 +105,11 @@ void *pl_fbdev_flip(void)
 	flip_cnt++;
 
 	if (pl_fbdev_buf != NULL) {
-		if (g_opts & OPT_SHOWFPS)
+		if (hud_msg[0] != 0)
+			print_hud();
+		else if (g_opts & OPT_SHOWFPS)
 			print_fps();
+
 		if (g_opts & OPT_SHOWCPU)
 			print_cpu_usage();
 	}
@@ -117,15 +134,25 @@ void pl_fbdev_close(void)
 static void update_input(void)
 {
 	int actions[IN_BINDTYPE_COUNT] = { 0, };
+	unsigned int emu_act;
 
 	in_update(actions);
-	if (actions[IN_BINDTYPE_EMU] & PEV_MENU)
-		stop = 1;
-	keystate = actions[IN_BINDTYPE_PLAYER12];
+	if (in_type == PSE_PAD_TYPE_ANALOGPAD)
+		in_update_analogs();
+	emu_act = actions[IN_BINDTYPE_EMU];
+	if (emu_act) {
+		int which = 0;
+		for (; !(emu_act & 1); emu_act >>= 1, which++)
+			;
+		emu_act = which;
+	}
+	emu_set_action(emu_act);
+
+	in_keystate = actions[IN_BINDTYPE_PLAYER12];
 
 #ifdef X11
-	extern void x11_update_keys(void);
-	x11_update_keys();
+	extern int x11_update_keys(void);
+	in_keystate |= x11_update_keys();
 #endif
 }
 
@@ -146,6 +173,7 @@ static void update_input(void)
 void pl_frame_limit(void)
 {
 	static struct timeval tv_old, tv_expect;
+	static int vsync_cnt_prev;
 	struct timeval now;
 	int diff;
 
@@ -162,12 +190,19 @@ void pl_frame_limit(void)
 		diff = tvdiff(now, tv_old);
 		vsps_cur = 0.0f;
 		if (0 < diff && diff < 2000000)
-			vsps_cur = 1000000.0f * vsync_cnt / diff;
+			vsps_cur = 1000000.0f * (vsync_cnt - vsync_cnt_prev) / diff;
+		vsync_cnt_prev = vsync_cnt;
 		flips_per_sec = flip_cnt;
-		vsync_cnt = flip_cnt = 0;
+		flip_cnt = 0;
 		tv_old = now;
 		if (g_opts & OPT_SHOWCPU)
 			tick_per_sec = get_cpu_ticks();
+
+		if (hud_new_msg > 0) {
+			hud_new_msg--;
+			if (hud_new_msg == 0)
+				hud_msg[0] = 0;
+		}
 	}
 #ifdef PCNT
 	static int ya_vsync_count;
@@ -177,18 +212,25 @@ void pl_frame_limit(void)
 	}
 #endif
 
-	if (!(g_opts & OPT_NO_FRAMELIM)) {
-		tvadd(tv_expect, pl_frame_interval);
-		diff = tvdiff(tv_expect, now);
-		if (diff > MAX_LAG_FRAMES * pl_frame_interval || diff < -MAX_LAG_FRAMES * pl_frame_interval) {
-			//printf("pl_frame_limit reset, diff=%d, iv %d\n", diff, pl_frame_interval);
-			tv_expect = now;
-		}
-		else if (diff > pl_frame_interval) {
-			// yay for working usleep on pandora!
-			//printf("usleep %d\n", diff - pl_frame_interval / 2);
-			usleep(diff - pl_frame_interval / 2);
-		}
+	tvadd(tv_expect, pl_frame_interval);
+	diff = tvdiff(tv_expect, now);
+	if (diff > MAX_LAG_FRAMES * pl_frame_interval || diff < -MAX_LAG_FRAMES * pl_frame_interval) {
+		//printf("pl_frame_limit reset, diff=%d, iv %d\n", diff, pl_frame_interval);
+		tv_expect = now;
+		diff = 0;
+	}
+
+	if (!(g_opts & OPT_NO_FRAMELIM) && diff > pl_frame_interval) {
+		// yay for working usleep on pandora!
+		//printf("usleep %d\n", diff - pl_frame_interval / 2);
+		usleep(diff - pl_frame_interval / 2);
+	}
+
+	plugin_skip_advice = 0;
+	if (UseFrameSkip && diff < -pl_frame_interval) {
+		// P.E.Op.S. makes skip decision based on this
+		fps_skip = 1.0f;
+		plugin_skip_advice = 1;
 	}
 
 	pcnt_start(PCNT_ALL);
@@ -238,14 +280,66 @@ static void pl_get_layer_pos(int *x, int *y, int *w, int *h)
 	*h = g_layer_h;
 }
 
-extern int UseFrameSkip; // hmh
-
 const struct rearmed_cbs pl_rearmed_cbs = {
 	pl_get_layer_pos,
 	pl_fbdev_open,
 	pl_fbdev_set_mode,
 	pl_fbdev_flip,
 	pl_fbdev_close,
-	&UseFrameSkip,
+	&plugin_skip_advice,
 };
 
+/* watchdog */
+static void *watchdog_thread(void *unused)
+{
+	int vsync_cnt_old = 0;
+	int seen_dead = 0;
+	int sleep_time = 5;
+
+#ifndef NDEBUG
+	// don't interfere with debug
+	return NULL;
+#endif
+	while (1)
+	{
+		sleep(sleep_time);
+
+		if (stop) {
+			seen_dead = 0;
+			sleep_time = 5;
+			continue;
+		}
+		if (vsync_cnt != vsync_cnt_old) {
+			vsync_cnt_old = vsync_cnt;
+			seen_dead = 0;
+			sleep_time = 2;
+			continue;
+		}
+
+		seen_dead++;
+		sleep_time = 1;
+		if (seen_dead > 1)
+			fprintf(stderr, "watchdog: seen_dead %d\n", seen_dead);
+		if (seen_dead > 4) {
+			fprintf(stderr, "watchdog: lockup detected, aborting\n");
+			// we can't do any cleanup here really, the main thread is
+			// likely touching resources and would crash anyway
+			abort();
+		}
+	}
+}
+
+void pl_start_watchdog(void)
+{
+	pthread_attr_t attr;
+	pthread_t tid;
+	int ret;
+	
+	pthread_attr_init(&attr);
+	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+	ret = pthread_create(&tid, &attr, watchdog_thread, NULL);
+	if (ret != 0)
+		fprintf(stderr, "could not start watchdog: %d\n", ret);
+}
+
